@@ -13,7 +13,6 @@
 #include "../src/getconfigoperation.h"
 #include "../src/mode.h"
 #include "../src/output.h"
-#include "../src/setconfigoperation.h"
 
 using namespace KScreen;
 
@@ -102,7 +101,7 @@ void TestModeListChange::cleanupTestCase()
 void TestModeListChange::modeListChange()
 {
     // json file for the fake backend
-    qputenv("KSCREEN_BACKEND_ARGS", "TEST_DATA=" TEST_DATA "singleoutput.json");
+    KScreen::BackendManager::instance()->setBackendArgs({{QStringLiteral("TEST_DATA"), TEST_DATA "singleoutput.json"}});
 
     const ConfigPtr config = getConfig();
     QVERIFY(!config.isNull());
