@@ -9,12 +9,15 @@
 #include <QObject>
 #include <QtTest>
 
+#include <cstdint>
+
 #include "../src/backendmanager_p.h"
 #include "../src/config.h"
 #include "../src/edid.h"
 #include "../src/getconfigoperation.h"
 #include "../src/mode.h"
 #include "../src/output.h"
+#include "../src/screen.h"
 
 Q_LOGGING_CATEGORY(KSCREEN_QSCREEN, "kscreen.qscreen")
 
@@ -159,7 +162,7 @@ void testQScreenBackend::commonUsagePattern()
 
         QVariantMap info;
         info[QStringLiteral("id")] = output->id();
-        info[QStringLiteral("primary")] = output->isPrimary();
+        info[QStringLiteral("priority")] = output->priority();
         info[QStringLiteral("enabled")] = output->isEnabled();
         info[QStringLiteral("rotation")] = output->rotation();
 
