@@ -5,10 +5,10 @@
  */
 
 #include "qscreenoutput.h"
-#include "qscreenbackend.h"
 
 #include <edid.h>
 #include <mode.h>
+#include <output.h>
 
 #include <QGuiApplication>
 
@@ -54,7 +54,6 @@ void QScreenOutput::updateKScreenOutput(OutputPtr &output) const
     // Initialize primary output
     output->setEnabled(true);
     output->setConnected(true);
-    output->setPrimary(QGuiApplication::primaryScreen() == m_qscreen);
 
     // Rotation - translate QScreen::primaryOrientation() to Output::rotation()
     if (m_qscreen->primaryOrientation() == Qt::PortraitOrientation) {
