@@ -6,11 +6,13 @@
  */
 #include "xrandrscreen.h"
 
-#include "../xcbwrapper.h"
-#include "config.h"
-#include "screen.h"
+#include "xrandr.h"
 #include "xrandrconfig.h"
 
+#include "../xcbwrapper.h"
+
+#include "screen.h"
+
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
 #include <private/qtx11extras_p.h>
 #else
@@ -32,7 +34,9 @@ XRandRScreen::~XRandRScreen()
 
 void XRandRScreen::update()
 {
-    xcb_screen_t *screen = XCB::screenOfDisplay(XCB::connection(), QX11Info::appScreen());
+    const int appScreen = QX11Info::appScreen();
+    m_id = appScreen;
+    xcb_screen_t *screen = XCB::screenOfDisplay(XCB::connection(), appScreen);
     m_currentSize = QSize(screen->width_in_pixels, screen->height_in_pixels);
 }
 
