@@ -10,7 +10,6 @@
 
 #include "abstractbackend.h"
 #include "backendinterface.h"
-#include "config.h"
 #include "configmonitor.h"
 #include "configserializer_p.h"
 #include "getconfigoperation.h"
@@ -56,6 +55,7 @@ BackendManager::BackendManager()
     , mShuttingDown(false)
     , mRequestsCounter(0)
     , mLoader(nullptr)
+    , mInProcessBackend(nullptr)
     , mMethod(OutOfProcess)
 {
     Log::instance();
@@ -169,12 +169,27 @@ QFileInfoList BackendManager::listBackends()
     const QStringList paths = QCoreApplication::libraryPaths();
     QFileInfoList finfos;
     for (const QString &path : paths) {
-        const QDir dir(path + QLatin1String("/kf5/kscreen/"), backendFilter, QDir::SortFlags(QDir::QDir::Name), QDir::NoDotAndDotDot | QDir::Files);
+        const QDir dir(path + QStringLiteral("/kf" QT_STRINGIFY(QT_VERSION_MAJOR) "/kscreen/"),
+                       backendFilter,
+                       QDir::SortFlags(QDir::QDir::Name),
+                       QDir::NoDotAndDotDot | QDir::Files);
         finfos.append(dir.entryInfoList());
     }
     return finfos;
 }
 
+void BackendManager::setBackendArgs(const QVariantMap &arguments)
+{
+    if (mBackendArguments != arguments) {
+        mBackendArguments = arguments;
+    }
+}
+
+QVariantMap BackendManager::getBackendArgs()
+{
+    return mBackendArguments;
+}
+
 KScreen::AbstractBackend *BackendManager::loadBackendPlugin(QPluginLoader *loader, const QString &name, const QVariantMap &arguments)
 {
     const auto finfo = preferredBackend(name);
@@ -210,28 +225,23 @@ KScreen::AbstractBackend *BackendManager::loadBackendInProcess(const QString &na
                               "loadBackendPlugin() instead.";
         return nullptr;
     }
-    if (m_inProcessBackend.first != nullptr && (name.isEmpty() || m_inProcessBackend.first->name() == name)) {
-        return m_inProcessBackend.first;
-    } else if (m_inProcessBackend.first != nullptr && m_inProcessBackend.first->name() != name) {
+    if (mInProcessBackend != nullptr && (name.isEmpty() || mInProcessBackend->name() == name)) {
+        return mInProcessBackend;
+    } else if (mInProcessBackend != nullptr && mInProcessBackend->name() != name) {
         shutdownBackend();
     }
 
     if (mLoader == nullptr) {
         mLoader = new QPluginLoader(this);
     }
-    auto test_data_equals = QStringLiteral("TEST_DATA=");
-    QVariantMap arguments;
-    auto beargs = QString::fromLocal8Bit(qgetenv("KSCREEN_BACKEND_ARGS"));
-    if (beargs.startsWith(test_data_equals)) {
-        arguments[QStringLiteral("TEST_DATA")] = beargs.remove(test_data_equals);
-    }
-    auto backend = BackendManager::loadBackendPlugin(mLoader, name, arguments);
+
+    auto backend = BackendManager::loadBackendPlugin(mLoader, name, mBackendArguments);
     if (!backend) {
         return nullptr;
     }
     // qCDebug(KSCREEN) << "Connecting ConfigMonitor to backend.";
     ConfigMonitor::instance()->connectInProcessBackend(backend);
-    m_inProcessBackend = qMakePair<KScreen::AbstractBackend *, QVariantMap>(static_cast<KScreen::AbstractBackend *>(backend), QVariantMap(arguments));
+    mInProcessBackend = backend;
     setConfig(backend->config());
     return backend;
 }
@@ -251,20 +261,7 @@ void BackendManager::requestBackend()
     }
     ++mRequestsCounter;
 
-    const QByteArray args = qgetenv("KSCREEN_BACKEND_ARGS");
-    QVariantMap arguments;
-    if (!args.isEmpty()) {
-        const QList<QByteArray> arglist = args.split(';');
-        for (const QByteArray &arg : arglist) {
-            const int pos = arg.indexOf('=');
-            if (pos == -1) {
-                continue;
-            }
-            arguments.insert(QString::fromUtf8(arg.left(pos)), arg.mid(pos + 1));
-        }
-    }
-
-    startBackend(QString::fromLatin1(qgetenv("KSCREEN_BACKEND")), arguments);
+    startBackend(QString::fromLatin1(qgetenv("KSCREEN_BACKEND")), mBackendArguments);
 }
 
 void BackendManager::emitBackendReady()
@@ -380,9 +377,8 @@ void BackendManager::shutdownBackend()
     if (mMethod == InProcess) {
         delete mLoader;
         mLoader = nullptr;
-        m_inProcessBackend.second.clear();
-        delete m_inProcessBackend.first;
-        m_inProcessBackend.first = nullptr;
+        delete mInProcessBackend;
+        mInProcessBackend = nullptr;
     } else {
         if (mBackendService.isEmpty() && !mInterface) {
             return;
