@@ -8,7 +8,6 @@
 #include "configserializer_p.h"
 
 #include "config.h"
-#include "edid.h"
 #include "kscreen_debug.h"
 #include "mode.h"
 #include "output.h"
@@ -19,6 +18,9 @@
 #include <QJsonDocument>
 #include <QRect>
 
+#include <cstdint>
+#include <optional>
+
 using namespace KScreen;
 
 QJsonObject ConfigSerializer::serializePoint(const QPoint &point)
@@ -79,7 +81,7 @@ QJsonObject ConfigSerializer::serializeOutput(const OutputPtr &output)
     obj[QLatin1String("connected")] = output->isConnected();
     obj[QLatin1String("followPreferredMode")] = output->followPreferredMode();
     obj[QLatin1String("enabled")] = output->isEnabled();
-    obj[QLatin1String("primary")] = output->isPrimary();
+    obj[QLatin1String("priority")] = static_cast<int>(output->priority());
     obj[QLatin1String("clones")] = serializeList(output->clones());
     // obj[QLatin1String("edid")] = output->edid()->raw();
     obj[QLatin1String("sizeMM")] = serializeSize(output->sizeMm());
@@ -224,6 +226,8 @@ ConfigPtr ConfigSerializer::deserializeConfig(const QVariantMap &map)
 OutputPtr ConfigSerializer::deserializeOutput(const QDBusArgument &arg)
 {
     OutputPtr output(new Output);
+    std::optional<bool> primary = std::nullopt;
+    std::optional<uint32_t> priority = std::nullopt;
 
     arg.beginMap();
     while (!arg.atEnd()) {
@@ -258,7 +262,13 @@ OutputPtr ConfigSerializer::deserializeOutput(const QDBusArgument &arg)
         } else if (key == QLatin1String("enabled")) {
             output->setEnabled(value.toBool());
         } else if (key == QLatin1String("primary")) {
-            output->setPrimary(value.toBool());
+            // primary is deprecated, but if it appears in config for compatibility reason.
+            primary = value.toBool();
+        } else if (key == QLatin1String("priority")) {
+            // "priority" takes precedence over "primary", but we need to
+            //  check it after the loop, otherwise it may come before the
+            //  primary and get overridden.
+            priority = value.toUInt();
         } else if (key == QLatin1String("clones")) {
             output->setClones(deserializeList<int>(value.value<QDBusArgument>()));
         } else if (key == QLatin1String("replicationSource")) {
@@ -293,6 +303,12 @@ OutputPtr ConfigSerializer::deserializeOutput(const QDBusArgument &arg)
         arg.endMapEntry();
     }
     arg.endMap();
+    if (primary.has_value()) {
+        output->setPriority(output->isEnabled() ? (primary.value() ? 1 : 2) : 0);
+    }
+    if (priority.has_value()) {
+        output->setPriority(priority.value());
+    }
     return output;
 }
 
