@@ -22,7 +22,7 @@ public:
     void trigger(KScreen::Dpms::Mode mode, const QList<QScreen *> &screens) override;
 
 private:
-    void blockUntilSupported() const override;
+    void blockUntilSupported() override;
 
     DpmsManager *m_dpmsManager = nullptr;
 };
