@@ -6,16 +6,18 @@
  */
 
 #include "output.h"
-#include "abstractbackend.h"
-#include "backendmanager_p.h"
 #include "edid.h"
-#include "kscreen_debug.h"
+#include "mode.h"
 
 #include <QCryptographicHash>
 #include <QGuiApplication>
 #include <QRect>
 #include <QScopedPointer>
 
+#include <cstdint>
+#include <qobjectdefs.h>
+#include <utility>
+
 using namespace KScreen;
 
 class Q_DECL_HIDDEN Output::Private
@@ -30,7 +32,7 @@ public:
         , explicitLogicalSize(QSizeF())
         , connected(false)
         , enabled(false)
-        , primary(false)
+        , priority(0)
         , edid(nullptr)
     {
     }
@@ -40,21 +42,21 @@ public:
         , name(other.name)
         , type(other.type)
         , icon(other.icon)
-        , clones(other.clones)
-        , replicationSource(other.replicationSource)
-        , currentMode(other.currentMode)
-        , preferredMode(other.preferredMode)
-        , preferredModes(other.preferredModes)
-        , sizeMm(other.sizeMm)
         , pos(other.pos)
         , size(other.size)
         , rotation(other.rotation)
-        , scale(other.scale)
-        , explicitLogicalSize(other.explicitLogicalSize)
+        , currentMode(other.currentMode)
+        , preferredMode(other.preferredMode)
+        , preferredModes(other.preferredModes)
         , connected(other.connected)
         , enabled(other.enabled)
-        , primary(other.primary)
+        , priority(other.priority)
+        , clones(other.clones)
+        , replicationSource(other.replicationSource)
+        , sizeMm(other.sizeMm)
+        , scale(other.scale)
         , followPreferredMode(other.followPreferredMode)
+        , explicitLogicalSize(other.explicitLogicalSize)
         , capabilities(other.capabilities)
         , overscan(other.overscan)
         , vrrPolicy(other.vrrPolicy)
@@ -72,32 +74,34 @@ public:
     QString biggestMode(const ModeList &modes) const;
     bool compareModeList(const ModeList &before, const ModeList &after);
 
+    // please keep them consistent with order of Q_PROPERTY declarations
     int id;
     QString name;
     Type type;
     QString icon;
     ModeList modeList;
-    QList<int> clones;
-    int replicationSource;
-    QString currentMode;
-    QString preferredMode;
-    QStringList preferredModes;
-    QSize sizeMm;
     QPoint pos;
     QSize size;
     Rotation rotation;
-    qreal scale;
-    QSizeF explicitLogicalSize;
+    // next three don't exactly match properties by name, but keep them close to each other anyway
+    QString currentMode;
+    QString preferredMode;
+    QStringList preferredModes;
+    //
     bool connected;
     bool enabled;
-    bool primary;
+    uint32_t priority;
+    QList<int> clones;
+    int replicationSource;
+    QScopedPointer<Edid> edid;
+    QSize sizeMm;
+    qreal scale;
     bool followPreferredMode = false;
+    QSizeF explicitLogicalSize;
     Capabilities capabilities;
     uint32_t overscan = 0;
     VrrPolicy vrrPolicy = VrrPolicy::Automatic;
     RgbRange rgbRange = RgbRange::Automatic;
-
-    QScopedPointer<Edid> edid;
 };
 
 bool Output::Private::compareModeList(const ModeList &before, const ModeList &after)
@@ -190,9 +194,7 @@ void Output::setId(int id)
     if (d->id == id) {
         return;
     }
-
     d->id = id;
-
     Q_EMIT outputChanged();
 }
 
@@ -206,9 +208,7 @@ void Output::setName(const QString &name)
     if (d->name == name) {
         return;
     }
-
     d->name = name;
-
     Q_EMIT outputChanged();
 }
 
@@ -240,9 +240,7 @@ void Output::setType(Type type)
     if (d->type == type) {
         return;
     }
-
     d->type = type;
-
     Q_EMIT outputChanged();
 }
 
@@ -293,9 +291,7 @@ void Output::setIcon(const QString &icon)
     if (d->icon == icon) {
         return;
     }
-
     d->icon = icon;
-
     Q_EMIT outputChanged();
 }
 
@@ -333,9 +329,7 @@ void Output::setCurrentModeId(const QString &mode)
     if (d->currentMode == mode) {
         return;
     }
-
     d->currentMode = mode;
-
     Q_EMIT currentModeIdChanged();
 }
 
@@ -367,7 +361,7 @@ QString Output::preferredModeId() const
     int total = 0;
     KScreen::ModePtr biggest;
     KScreen::ModePtr candidateMode;
-    for (const QString &modeId : qAsConst(d->preferredModes)) {
+    for (const QString &modeId : std::as_const(d->preferredModes)) {
         candidateMode = mode(modeId);
         const int area = candidateMode->size().width() * candidateMode->size().height();
         if (area < total) {
@@ -406,9 +400,7 @@ void Output::setPos(const QPoint &pos)
     if (d->pos == pos) {
         return;
     }
-
     d->pos = pos;
-
     Q_EMIT posChanged();
 }
 
@@ -422,9 +414,7 @@ void Output::setSize(const QSize &size)
     if (d->size == size) {
         return;
     }
-
     d->size = size;
-
     Q_EMIT sizeChanged();
 }
 
@@ -439,9 +429,7 @@ void Output::setRotation(Output::Rotation rotation)
     if (d->rotation == rotation) {
         return;
     }
-
     d->rotation = rotation;
-
     Q_EMIT rotationChanged();
 }
 
@@ -483,9 +471,7 @@ void Output::setConnected(bool connected)
     if (d->connected == connected) {
         return;
     }
-
     d->connected = connected;
-
     Q_EMIT isConnectedChanged();
 }
 
@@ -499,26 +485,32 @@ void Output::setEnabled(bool enabled)
     if (d->enabled == enabled) {
         return;
     }
-
     d->enabled = enabled;
-
     Q_EMIT isEnabledChanged();
 }
 
 bool Output::isPrimary() const
 {
-    return d->primary;
+    return d->enabled && (d->priority == 1);
 }
 
 void Output::setPrimary(bool primary)
 {
-    if (d->primary == primary) {
+    setPriority(1);
+}
+
+uint32_t Output::priority() const
+{
+    return d->priority;
+}
+
+void Output::setPriority(uint32_t priority)
+{
+    if (d->priority == priority) {
         return;
     }
-
-    d->primary = primary;
-
-    Q_EMIT isPrimaryChanged();
+    d->priority = priority;
+    Q_EMIT priorityChanged();
 }
 
 QList<int> Output::clones() const
@@ -531,9 +523,7 @@ void Output::setClones(const QList<int> &outputlist)
     if (d->clones == outputlist) {
         return;
     }
-
     d->clones = outputlist;
-
     Q_EMIT clonesChanged();
 }
 
@@ -547,9 +537,7 @@ void Output::setReplicationSource(int source)
     if (d->replicationSource == source) {
         return;
     }
-
     d->replicationSource = source;
-
     Q_EMIT replicationSourceChanged();
 }
 
@@ -581,10 +569,11 @@ bool KScreen::Output::followPreferredMode() const
 
 void KScreen::Output::setFollowPreferredMode(bool follow)
 {
-    if (follow != d->followPreferredMode) {
-        d->followPreferredMode = follow;
-        Q_EMIT followPreferredModeChanged(follow);
+    if (follow == d->followPreferredMode) {
+        return;
     }
+    d->followPreferredMode = follow;
+    Q_EMIT followPreferredModeChanged(follow);
 }
 
 bool Output::isPositionable() const
@@ -621,10 +610,11 @@ Output::Capabilities Output::capabilities() const
 
 void Output::setCapabilities(Capabilities capabilities)
 {
-    if (d->capabilities != capabilities) {
-        d->capabilities = capabilities;
-        Q_EMIT capabilitiesChanged();
+    if (d->capabilities == capabilities) {
+        return;
     }
+    d->capabilities = capabilities;
+    Q_EMIT capabilitiesChanged();
 }
 
 uint32_t Output::overscan() const
@@ -634,10 +624,11 @@ uint32_t Output::overscan() const
 
 void Output::setOverscan(uint32_t overscan)
 {
-    if (d->overscan != overscan) {
-        d->overscan = overscan;
-        Q_EMIT overscanChanged();
+    if (d->overscan == overscan) {
+        return;
     }
+    d->overscan = overscan;
+    Q_EMIT overscanChanged();
 }
 
 Output::VrrPolicy Output::vrrPolicy() const
@@ -647,10 +638,11 @@ Output::VrrPolicy Output::vrrPolicy() const
 
 void Output::setVrrPolicy(VrrPolicy policy)
 {
-    if (d->vrrPolicy != policy) {
-        d->vrrPolicy = policy;
-        Q_EMIT vrrPolicyChanged();
+    if (d->vrrPolicy == policy) {
+        return;
     }
+    d->vrrPolicy = policy;
+    Q_EMIT vrrPolicyChanged();
 }
 
 Output::RgbRange Output::rgbRange() const
@@ -660,10 +652,11 @@ Output::RgbRange Output::rgbRange() const
 
 void Output::setRgbRange(Output::RgbRange rgbRange)
 {
-    if (d->rgbRange != rgbRange) {
-        d->rgbRange = rgbRange;
-        Q_EMIT rgbRangeChanged();
+    if (d->rgbRange == rgbRange) {
+        return;
     }
+    d->rgbRange = rgbRange;
+    Q_EMIT rgbRangeChanged();
 }
 
 void Output::apply(const OutputPtr &other)
@@ -674,8 +667,7 @@ void Output::apply(const OutputPtr &other)
     // We block all signals, and emit them only after we have set up everything
     // This is necessary in order to prevent clients from accessing inconsistent
     // outputs from intermediate change signals
-    const bool keepBlocked = signalsBlocked();
-    blockSignals(true);
+    const bool keepBlocked = blockSignals(true);
     if (d->name != other->d->name) {
         changes << &Output::outputChanged;
         setName(other->d->name);
@@ -712,9 +704,9 @@ void Output::apply(const OutputPtr &other)
         changes << &Output::isEnabledChanged;
         setEnabled(other->d->enabled);
     }
-    if (d->primary != other->d->primary) {
-        changes << &Output::isPrimaryChanged;
-        setPrimary(other->d->primary);
+    if (d->priority != other->d->priority) {
+        changes << &Output::priorityChanged;
+        setPriority(other->d->priority);
     }
     if (d->clones != other->d->clones) {
         changes << &Output::clonesChanged;
@@ -775,10 +767,22 @@ QDebug operator<<(QDebug dbg, const KScreen::OutputPtr &output)
         return dbg;
     }
 
-    dbg.nospace() << "KScreen::Output(" << output->id() << ", " << output->name() << ", " << (output->isConnected() ? "connected " : "disconnected ")
-                  << (output->isEnabled() ? "enabled" : "disabled") << (output->isPrimary() ? " primary" : "") << ", pos: " << output->pos()
-                  << ", res: " << output->size() << ", modeId: " << output->currentModeId() << ", scale: " << output->scale()
-                  << ", clone: " << (output->clones().isEmpty() ? "no" : "yes") << ", rotation: " << output->rotation()
-                  << ", followPreferredMode: " << output->followPreferredMode() << ")";
+    // clang-format off
+    dbg.nospace()
+        << "KScreen::Output("
+            << output->id() << ", "
+            << output->name() << ", "
+            << (output->isConnected() ? "connected " : "disconnected ")
+            << (output->isEnabled() ? "enabled" : "disabled")
+            << " priority " << output->priority()
+            << ", pos: " << output->pos()
+            << ", res: " << output->size()
+            << ", modeId: " << output->currentModeId()
+            << ", scale: " << output->scale()
+            << ", clone: " << (output->clones().isEmpty() ? "no" : "yes")
+            << ", rotation: " << output->rotation()
+            << ", followPreferredMode: " << output->followPreferredMode()
+        << ")";
+    // clang-format on
     return dbg;
 }
